(ns clojsa.saparser
  (:require [clojure.string :as string]
            [clojure.pprint :refer [pprint]]
            [hickory.core :refer :all]
            [hickory.select :as s]
            [hickory.convert :refer [hickory-to-hiccup]]))

(defn hickory-doc [doc]
  (-> doc parse as-hickory))

(defn hickory-div [content class]
  {:type :element
   :attrs {:class class}
   :tag :div
   :content content})

(defn parse-title [htree]
  (-> (s/select (s/child (s/tag :title)) htree)
      first :content first
      (string/replace #" - The Something Awful Forums" "")))

(defn parse-pagecount [htree]
  (-> (s/select (s/descendant
                 (s/class :pages) (s/tag :option)) htree)
      last :content first Integer/parseInt))

(defn parse-thread [htree]
  (-> (s/select (s/descendant
                 (s/id :thread))
                htree)
      first))

(defn select-td [class-key htree]
  (s/select (s/descendant
             (s/and (s/tag :td) (s/class class-key))) htree))

(defn parse-ui [ui]
  (let [ui (first (s/select (s/descendant (s/tag :dl)) ui))
        author (-> (s/select (s/descendant (s/class :author)) ui)
                   first :content first)
        regdate (-> (s/select (s/descendant (s/class :registered)) ui)
                    first :content first)
        title (-> (s/select (s/descendant (s/class :title)) ui) first)
        avatar (-> (s/select (s/descendant (s/tag :img)) title) first)]
    {:author author
     :regdate regdate
     :avatar-title (hickory-to-hiccup
                    (hickory-div (:content title) "avatar-title"))
     :avatar (when avatar (hickory-to-hiccup avatar))}))

(defn parse-pd [pd]
  (string/trim (last (hickory-to-hiccup pd))))

(defn parse-pb [pb]
  (let [pb (-> pb :content)]
    (hickory-to-hiccup (hickory-div pb "postbody"))))

(defn thread-map [id page doc]
  (let [htree       (hickory-doc doc)
        title       (parse-title htree)
        page-count  (parse-pagecount htree)
        thread-tree (parse-thread htree)
        userinfo    (select-td :userinfo thread-tree)
        postdate    (select-td :postdate thread-tree)
        postbody    (select-td :postbody thread-tree)]
    {:title title
     :id id
     :page page
     :page-count page-count
     :content
     (for [[ui pd pb] (partition 3 (interleave userinfo postdate postbody))
           :when (not= "Adbot" (-> (s/select (s/child (s/class :author)) ui)
                                   first :content first))]
       {:ui (parse-ui ui) :pd (parse-pd pd) :pb (parse-pb pb)})}))

(defn parse-bookmarks [htree]
  (s/select (s/descendant
             (s/id :forum)
             (s/tag :tbody)
             (s/tag :tr))
            htree))

(defn parse-row [htree]
  (when-let [link (-> (s/select (s/descendant (s/class :info) (s/tag :a)) htree) first)]
    (let [thread-id (re-find #"\d+$" (:href (:attrs link)))
          title (-> link :content first string/trim)]
      {:id thread-id :title  title})))

(defn bookmarks-map [page doc]
  (let [htree       (hickory-doc doc)
        title       (parse-title htree)
        page-count  (parse-pagecount htree)
        bookmarks-tree (parse-bookmarks htree)]
    {:title title
     :page page
     :page-count page-count
     :content
     (for [row bookmarks-tree
           :let [parsed-row (parse-row row)]
           :when parsed-row]
       parsed-row)}))
